package quest.client;

import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.util.*;
import java.net.*;
import java.io.IOException;
import java.awt.event.KeyEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import quest.global.chat.*;
import quest.client.net.ClientDaemon;
import quest.global.net.MessageClientListener;
import quest.global.net.Message;
import quest.global.login.LoginMessage;
import quest.global.util.FiniteStateMachine;   // MATHIJS: FiniteStateMachine zit nu in quest.global.util

/*

TODO:
+fix als rankingtable null userranking terugkrijgt
+last updated info in ranking fixen
+alle sjit bij tables right alignen
+auto update persistent maken en default uit
+helpwindow maken (JHTMLPane??)
+info en whisper knoppen fixen
+move to spectators/players fixen
+search implementeren
+admin vinkje maken in chat bij players

+fix singleton constraint in applet
+start game/join game enablement checken
+leavegame chat tab enablement fixen
+election forcen clientside (is nodig kennelijk)
+clientside connectie afsluiten als inloggen niet succesvol was


+netjes uitloggen bij exit
	+subprocessen uitloggen logisch niet feasable
		+wel geimplementeerd
	+na 3s geen response naar begin toestand raggen
+up/downarrow fixen bij playeradmin
*/

/**
 * <p>The Applet is the entry point for the user, where he or she can log into the game.
 * It instantiates ClientFrame when propperly logged in. </p>
 * This class is an example of a <a href="../../../overview-summary.html#patternSingleton">Singleton design pattern</a>.
 *
 * @author Sander Brandenburg
 */
public class ClientApplet extends JApplet implements MessageClientListener {
    private Image				questLogo;
    private Image				questPreloader;
    private Dimension			questLogoDim;
    private Dimension			questPreloaderDim;
    private JTextField		usernameField;
    private JPasswordField	passwordField;
    private JLabel			resultLabel;
    private JButton			loginButton;
    private JButton			newuserButton;
	
    private GridBagLayout gbl = new GridBagLayout();
	
    private ClientDaemon cd;	

    final static Dimension frameSize = new Dimension(640, 480);

    private final static int STATE_BEGIN			= 0;
    private final static int STATE_LOGIN			= 1;
    private final static int STATE_NEWUSER			= 2;
    private final static int STATE_LOGGEDIN			= 3;
    private final static int STATE_EXIT				= 4;
    // if a second clientapplet is started in the same jvm address space, dead state
    private final static int STATE_SINGLETONFAULT	= 5;
	
    private final static int NR_STATES				= 6;
	
    private final static int transitions[][] = {
	{STATE_BEGIN, STATE_LOGIN},
	{STATE_BEGIN, STATE_NEWUSER},
	{STATE_LOGIN, STATE_LOGGEDIN},
	{STATE_NEWUSER, STATE_LOGGEDIN},
	{STATE_LOGIN, STATE_BEGIN},
	{STATE_NEWUSER, STATE_BEGIN},
	{STATE_LOGGEDIN, STATE_EXIT},
	{STATE_EXIT, STATE_BEGIN},
    };
	
    private FiniteStateMachine fsm = new FiniteStateMachine(NR_STATES, transitions);

    private String exitMsg;
    private ClientFrame jf;
	
    private static ClientApplet uniqueInstance;

    /**
     * The entry point for the applet. Initializes the form where user can enter username and password.
     */
    public void init() {
	System.out.println("init");
		
	if (uniqueInstance != null) {
	    fsm.start(STATE_SINGLETONFAULT);
	    initFormSingletonFault();
	    return;
	} 

	uniqueInstance = this;
	fsm.start(STATE_BEGIN);

	questLogo = getImage(getCodeBase(), "quest/client/images/questlogo.png");
	questPreloader = getImage(getCodeBase(), "quest/client/images/preloader.png");
		
	MediaTracker mt = new MediaTracker(this);
	mt.addImage(questLogo, 0);
	mt.addImage(questPreloader, 0);
	try {
	    mt.waitForID(0);
	} catch (InterruptedException e) {}
		
	questLogoDim = new Dimension(questLogo.getWidth(this), questLogo.getHeight(this));
	questPreloaderDim = new Dimension(questPreloader.getWidth(this), questPreloader.getHeight(this));
				
	initFormBegin();
    }
	
    /**
     * Returns the instance of the clientApplet.
     * @return ClientApplet The clientapplet
     * @exception NullPointerException When not initialized.
     */
    public static ClientApplet getInstance() {
	if (uniqueInstance == null)
	    throw new RuntimeException("ClientApplet not initialized!");
	return uniqueInstance;
    }
	
    /**
     * Only requests focus when not logged in yet.
     */
    public void start() {
	System.out.println("start");
		
	if (fsm.getCurrentState() == STATE_BEGIN) {
	    if (loginButton != null) {
		loginButton.requestFocus();
		loginButton.grabFocus();
	    }
	}
    }

    /**
     * Terminates applet.
     */
    public void stop() {
	System.out.println("stop");
	if (uniqueInstance == this) { // check if it's the original instance
	    uniqueInstance = null;	  
	}
    }
	
    private void setEnabledInput(boolean flag) {
	loginButton.setEnabled(flag);
	newuserButton.setEnabled(flag);
		
	passwordField.setEnabled(flag);
	usernameField.setEnabled(flag);
    }
	
    private void setPanel(JPanel pan) {

	Container cp = getContentPane();
	cp.removeAll();
	cp.add(pan, BorderLayout.CENTER);
	cp.validate();
	//getRootPane().setContentPane(pan);
    }
	
    private void initFormBegin() {
	JPanel cp = new JPanel();
	GridBagLayout gbl = new GridBagLayout();
	cp.setLayout(gbl);
		
	GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(4, 4, 4, 4);
		
	JLabel icon = new JLabel(new ImageIcon(questLogo));
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.CENTER;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.weightx = 0.0;
	c.weighty = 0.0;
	gbl.setConstraints(icon, c);
	c.gridwidth = 1;
	cp.add(icon);
		
	JLabel lab1 = new JLabel("Username");
	lab1.setDisplayedMnemonic(KeyEvent.VK_U);
	c.gridx = 0;
	c.gridy = 1;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	c.weightx = 0.0;
	c.weighty = 0.0;
	c.insets.left = 10;
	gbl.setConstraints(lab1, c);
	c.insets.left = 4;
	cp.add(lab1);
		
	JLabel lab2 = new JLabel("Password");
	lab1.setDisplayedMnemonic(KeyEvent.VK_P);
	c.gridx = 0;
	c.gridy = 2;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	c.weightx = 0.0;
	c.weighty = 0.0;
	c.insets.left = 10;
	gbl.setConstraints(lab2, c);
	c.insets.left = 4;
	cp.add(lab2);
		
	usernameField = new JTextField("", 32);
	lab1.setLabelFor(usernameField);
	c.gridx = 1;
	c.gridy = 1;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = 1.0;
	c.weighty = 0.0;
	c.insets.right = 10;
	gbl.setConstraints(usernameField, c);
	c.insets.right = 4;
	cp.add(usernameField);
		
	passwordField = new JPasswordField("", 32);
	lab2.setLabelFor(passwordField);
	c.gridx = 1;
	c.gridy = 2;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = 1.0;
	c.weighty = 0.0;
	c.insets.right = 10;
	gbl.setConstraints(passwordField, c);
	c.insets.right = 4;
	cp.add(passwordField);
		
	JPanel jp1 = new JPanel();
	jp1.setLayout(new BoxLayout(jp1, BoxLayout.X_AXIS));
	loginButton = new JButton("Log in");
	loginButton.setMnemonic(KeyEvent.VK_L);
	loginButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    if (!fsm.validNextState(STATE_LOGIN)) {
			return;
		    }
		    if (!internalCheckUserPass())
			return;
			
		    final Runnable run = new Runnable() {
			    public void run() {
				login();
			    }
			};
		    new Thread(run).start();
		}
	    });
	getRootPane().setDefaultButton(loginButton);
		
	JButton newuser = newuserButton = new JButton("New User");
	newuser.setMnemonic(KeyEvent.VK_N);
	newuser.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    if (!fsm.validNextState(STATE_NEWUSER)) {
			return;
		    }			
		    if (!internalCheckUserPass())
			return;
			
		    final Runnable run = new Runnable() {
			    public void run() {
				newuser();
			    }
			};
		    new Thread(run).start();
		}
	    });

	jp1.add(Box.createHorizontalGlue());
	jp1.add(loginButton);
	jp1.add(Box.createHorizontalGlue());
	jp1.add(Box.createRigidArea(new Dimension(5, 0)));
	jp1.add(Box.createHorizontalGlue());
	jp1.add(newuser);
	jp1.add(Box.createHorizontalGlue());
	c.gridx = 0;
	c.gridy = 3;
	c.gridwidth = 2;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = 1.0;
	c.weighty = 0.0;
	c.insets.right	= 10;
	c.insets.left	= 10;
	c.insets.top	= 10;
	c.insets.bottom = 10;
	gbl.setConstraints(jp1, c);
	cp.add(jp1);
		
	resultLabel = new JLabel(" ");
	resultLabel.setFont(new Font("Tahoma", Font.BOLD, 12));
	resultLabel.setForeground(Color.black);
	c.gridx = 0;
	c.gridy = 4;
	c.gridwidth = 2;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = 1.0;
	c.weighty = 0.0;
	c.insets.right	= 10;
	c.insets.left	= 10;
	c.insets.top	= 5;
	c.insets.bottom = 5;
	gbl.setConstraints(resultLabel, c);
	cp.add(resultLabel);
		
	setPanel(cp);
    }
	
    private void initFormLoggedIn() {
	JPanel pan = new JPanel();
		
	pan.setLayout(new BorderLayout());
	JLabel lab = new JLabel(new ImageIcon(questPreloader));
	pan.add(lab, BorderLayout.CENTER);
				
	setPanel(pan);
    }
	
    private void initFormExit(String msg) {
	JPanel pan = new JPanel();
	GridBagLayout gbl = new GridBagLayout();
		
	pan.setLayout(gbl);
	GridBagConstraints c = new GridBagConstraints();
		
	JTextArea jta = new JTextArea(msg);
	//jta.setLineWrap(true);
	//jta.setWrapStyleWord(true);
	jta.setEditable(false);
	jta.setHighlighter(null);
	jta.setBorder(BorderFactory.createEtchedBorder());

	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.CENTER;
	gbl.setConstraints(jta, c);
	pan.add(jta);
		
	JButton again = new JButton("Play Again");
	again.setMnemonic(KeyEvent.VK_A);
	again.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    fsm.setState(STATE_BEGIN);
		    initFormBegin();
		}
	    });
	c.gridy = 1;
	c.anchor = GridBagConstraints.NORTH;
	gbl.setConstraints(again, c);
	pan.add(again);

	setPanel(pan);
	getRootPane().setDefaultButton(again);
    }
	
    private void initFormSingletonFault() {
		
	JPanel pan = new JPanel();
	GridBagLayout gbl = new GridBagLayout();
	pan.setLayout(gbl);
		
	GridBagConstraints c = new GridBagConstraints();
		
	JTextArea jta = new JTextArea("This applet can only be run\nonce in the same address space");
	//jta.setLineWrap(true);
	//jta.setWrapStyleWord(true);
	jta.setEditable(false);
	jta.setHighlighter(null);
	jta.setBorder(BorderFactory.createEtchedBorder());
	c.gridx = 0;
	c.gridy = 0;
	c.anchor = GridBagConstraints.CENTER;
	c.fill = GridBagConstraints.NONE;
	c.weightx = 1.0;
	c.weighty = 1.0;
	gbl.setConstraints(jta, c);
	pan.add(jta);
				
	setPanel(pan);
    }
	
    private void newuser() {
	fsm.setState(STATE_NEWUSER);
	setEnabledInput(false);
		
	String username = usernameField.getText();
	String password = new String(passwordField.getPassword());
	resultLabel.setText("connecting");

	initClientDaemon();
	if (cd == null) {
	    fsm.setState(STATE_BEGIN);
	    resultLabel.setText("couldn't set up connection");
	    setEnabledInput(true);
	    return;
	}
		
	LoginMessage.C2S_NewUser newuser = new LoginMessage.C2S_NewUser(username, password);
	cd.sendMessage(newuser);
	resultLabel.setText("registering new user");
    }
	
    private void login() {
	fsm.setState(STATE_LOGIN);
	setEnabledInput(false);
		
	usernameField.setEnabled(false);
	passwordField.setEnabled(false);
		
	String username = usernameField.getText();
	String password = new String(passwordField.getPassword());

	resultLabel.setText("connecting");
	initClientDaemon();
	if (cd == null) {
	    fsm.setState(STATE_BEGIN);
	    //resultLabel.setText("couldn't set up connection");
	    setEnabledInput(true);
	    return;
	}
		
	LoginMessage.C2S_Login login = new LoginMessage.C2S_Login(username, password);
	cd.sendMessage(login);
	resultLabel.setText("logging in");
    }	
    //hack: delete username/password
    private void initClientDaemon() {
	try {
	    if (cd != null) {
		cd.close();
		cd = null;
	    }

	    cd = new ClientDaemon(InetAddress.getByName(getCodeBase().getHost()));
	} catch (UnknownHostException uhe) {
	    resultLabel.setText("unknown host");
	    return;
	} catch (IOException ie) {
	    resultLabel.setText("can't connect");
	    return;
	}
	cd.registerProces(LoginMessage.class, this);
    }	

  /**
   * Messagehandler called by ClientDaemon.
   * @param msg The LoginMessage.
   * @see quest.global.chat.SearchMessage
   * @exception RuntimeException when msg is not a LoginMessage.
   */	
    public void incomingMessage(Message msg) {
	if (!(msg instanceof LoginMessage)) {
	    throw new RuntimeException("msg not LoginMessage!");
	}

	LoginMessage lm = (LoginMessage) msg;
	if (lm instanceof LoginMessage.S2C_AllowLogin) {
	    if (!fsm.validNextState(STATE_LOGGEDIN)) {
		fsm.setState(STATE_BEGIN);
		initFormBegin();
		resultLabel.setText("unexpected packet received");
		return;
	    }
	    fsm.setState(STATE_LOGGEDIN);
	    initFormLoggedIn();
	    resultLabel.setText("login accepted");
	    spawnFrame();
	} else if (lm instanceof LoginMessage.S2C_DenyLogin) {
	    if (!fsm.validNextState(STATE_BEGIN)) {
		fsm.setState(STATE_BEGIN);
		initFormBegin();
		resultLabel.setText("login denied");
		return;				
	    }
			
	    fsm.setState(STATE_BEGIN);
	    LoginMessage.S2C_DenyLogin deny = (LoginMessage.S2C_DenyLogin) lm;
	    resultLabel.setText(deny.explanation);
	    setEnabledInput(true);
	    cd.close();
	} else if (lm instanceof LoginMessage.S2C_Exit) {
	    if (!fsm.validNextState(STATE_EXIT)) {
		fsm.setState(STATE_BEGIN);
		initFormBegin();
		resultLabel.setText("login denied");
		return;				
	    }
			
	    // check for "exiter" threads and kill them
	    // this can't be done in the exit method because
	    // the "exiter" thread would call the exit method directly
			
	    Thread[] tarray = new Thread[Thread.activeCount()];
	    Thread.enumerate(tarray);
	    for (int i = 0; i < tarray.length; i++) {
		if (tarray[i].getName().equals("exiter")) {
		    tarray[i].interrupt();
		    System.out.println("exiter thread " + i + " interrupted");
		}
	    }

	    LoginMessage.S2C_Exit exit = (LoginMessage.S2C_Exit) lm;
	    exit(exit.exitMessage);
	}
    }

    /**
     * Returns whether user is logged in correctly.
     * @return boolean whether user is logged in correctly.
     */
    public boolean isLoggedIn() {
	return fsm.getCurrentState() == STATE_LOGGEDIN;
    }
	
    /**
     * Sets the applet in 'exit' mode, where the user can see some statistics about the previous session and can press 'play again' button.
     */
    public void exit(String msg) {
	//	System.out.println("exit("+msg+") called - " + Thread.currentThread().getName());
	fsm.setState(STATE_EXIT);
	initFormExit(msg);
		
	if (cd != null)
	    cd.close();
			
	if (jf != null) {
	    jf.setVisible(false);
	    jf.dispose();
	    jf = null;
	}
    }
	
    private boolean internalCheckUserPass() {	
	String username = usernameField.getText();
	String password = new String(passwordField.getPassword());
	if (username.trim().length() < 4) {
	    resultLabel.setText("username too short");
	} else if (password.trim().length() < 4) {
	    resultLabel.setText("password too short");
	} else {
	    return true;
	}
	return false;
    }
	
    private void spawnFrame() {
	String username = usernameField.getText();
	String password = new String(passwordField.getPassword());
		
	jf = new ClientFrame(username, password, cd, getCodeBase());		
	jf.setSize(frameSize);
	jf.setResizable(true);
		
	Dimension screenSize = getToolkit().getScreenSize();
	if (frameSize.height > screenSize.height)
	    frameSize.height = screenSize.height;
	if (frameSize.width > screenSize.width)
	    frameSize.width = screenSize.width;
		
	jf.setLocation((screenSize.width - frameSize.width) / 2, 
		       (screenSize.height - frameSize.height) / 2);
	jf.setVisible(true);
    }

    /**
     * External interface used by design tools to show properties of an applet.
     * TODO: NYI
     */
    /*
      public String[][] getParameterInfo() {
      String[][] info =
      {
      //			{ labelParam, "String", "Label string to be displayed" },
      //			{ backgroundParam, "String", "Background color, format \"rrggbb\"" },
      //			{ foregroundParam, "String", "Foreground color, format \"rrggbb\"" },
      };
      return info;
      }
    */
	
    /**
     * Returns applet info.
     */
    public String getAppletInfo() {
	return "Title: Amazing Quest \n"
	    + "Author: Sander Brandenburg, Mathijs den Burger and Merijn Evertse\n"
	    + "3D boardgame. Needs java3d and javaplugin.";
    }


    /*private String[] names =  { "tryfont.ttf" };
      private static Hashtable fontCache;


      void initFonts() {
      fontCache = new Hashtable(names.length);
      for (int i = 0; i < names.length; i++) {
      fontCache.put(names[i], getFont(names[i]));
      }
      }
	
      public static Font getFont(String name) {
      Font font = null;
      if (fontCache != null) {
      if ((font = (Font) fontCache.get(name)) != null) {
      return font;
      }
      }
      String fName = "fonts/" + name;
      InputStream is = null;
      try {
      URL url = new URL(applet.getCodeBase(), fName);
      is = url.openStream();
      font = Font.createFont(Font.TRUETYPE_FONT, is);
      } catch (Exception e) { 
      e.printStackTrace(); 
      System.err.println(fName + " not loaded.  Using serif font.");
      font = new Font("serif", Font.PLAIN, 24);
      } finally {
      if (is != null)
      is.close();
      }
      return font;
      }*/
	
}
