package quest.client.chat;

import java.awt.event.KeyEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.*;
import java.util.Vector;
import java.awt.*;
import quest.global.chat.*;

import quest.client.ClientFrame;
import quest.global.net.*;
import quest.client.net.*;

import javax.swing.*;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * Represents the Channel tab in the Chat GUI.
 * Here the user can retrieve an overview of all the channels that are active on the server.
 * In this overview it's possible to join a channel using the popupmenu.
 * <p>
 * There are two modes of channel overviews: connection oriented(using 'autoupdate') or connectionless(using the 'Update' button). The connection oriented overview update is pushed by the server. The connectionless overview has to be  pulled by the user, clientside using the 'Update' button.
 * </p>
 *
 * @author Sander Brandenburg
 */
public class GUITab_Channel extends JPanel implements MessageClientListener {
	
  private final String[] columnNames = {
	"Game name", "Name Operator", "State", "Players", "Spectators"
  };
	
  private final Class[] classTypes = {
	String.class, String.class, String.class, Integer.class, Integer.class
  };

  private final double leftwindow_weightx = 1.0;
  private final Insets insets_none = new Insets(0, 0, 0, 0);
  private final Insets insets_small = new Insets(2, 2, 2, 2);
  private final ClientDaemon cd = ClientDaemon.getInstance();

  /**
   * The channel overview is represented as a RankingTable.
   */
  protected RankingTable rt;
  /**
   * The channel data retrieved from the server, when available.
   */
  protected Vector channelData;
	
  /**
   * Represents the autoUpdate checkbox.
   */
  protected JCheckBox autoUpdateCheckBox;

  /**
   * The 'create channel' button.
   */
  protected JButton createChannel;
  /**
   * The 'join Channel' menuitem.
   */
  protected JMenuItem joinMenuItem;

	
  /**
   * Toggles whether the Join Channel button is enabled.
   */
  public void setEnabledJoinChannel(boolean flag) {
	joinMenuItem.setEnabled(flag);
  }

  /**
   * Toggles whether the Create Channel button is enabled.
   */
  public void setEnabledCreateChannel(boolean flag) {
	createChannel.setEnabled(flag);
  }

  /**
   * Calls initForm and registers ChannelMessage to the ClientDaemon.
   */
  public GUITab_Channel() {
	initForm();

	cd.registerProces(ChannelMessage.class, this);
  }
	
  /**
   * Sends a subscribe ChannelMessage to the server.
   */
  protected void subscribe() {
	ChannelMessage.C2S_Subscribe sub = new ChannelMessage.C2S_Subscribe();
	cd.sendMessage(sub);
  }
	
  /**
   * Sends an unsubscribe ChannelMessage to the server.
   */
  protected void unsubscribe() {
	ChannelMessage.C2S_UnSubscribe unsub = new ChannelMessage.C2S_UnSubscribe();
	cd.sendMessage(unsub);
  }

  /**
   * Sends a connectionless channel update request.
   */
  protected void request() {
	ChannelMessage.C2S_Request req = new ChannelMessage.C2S_Request();
	cd.sendMessage(req);
  }

  /**
   * Initializes the form.
   */
  void initForm() {
	GridBagLayout gbl = new GridBagLayout();
	setLayout(gbl);
		
	GridBagConstraints c = new GridBagConstraints();
		
	JLabel lab1 = new JLabel("Available Channels");
	lab1.setDisplayedMnemonic(KeyEvent.VK_A);
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.SOUTHWEST;
	c.weightx = leftwindow_weightx;
	c.weighty = 0.0;
	c.insets = insets_small;
	gbl.setConstraints(lab1, c);
	add(lab1);
		
	JLabel lab2 = new JLabel("auto update");
		
	final JButton updateb = new JButton("Update");
		
	autoUpdateCheckBox = new JCheckBox();
	autoUpdateCheckBox.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent ae) {
		Runnable run = new Runnable() {
		  public void run() {
			if (autoUpdateCheckBox.isSelected()) {
			  subscribe();
			  updateb.setEnabled(false);
			} else {
			  unsubscribe();
			  updateb.setEnabled(true);
			}
		  }
		};
		(new Thread(run)).start();
	  }
	});
		
	updateb.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent ae) {
		Runnable run = new Runnable() {
		  public void run() {
			request();
		  }
		};
		(new Thread(run)).start();
	  }
	});
		
	channelData = new Vector(100);
	VectorTableModel tm = new VectorTableModel(columnNames, classTypes, channelData);
	rt = new RankingTable("loading channels");
	rt.setTableMode(tm, true);
		
	// begin bouwen popupmenu	
	final StringBuffer target = new StringBuffer(40);
	final JPopupMenu pum = new JPopupMenu();
	JMenuItem mi = joinMenuItem = new JMenuItem("Join Channel", KeyEvent.VK_J);
	mi.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent ae) {
		if (target.length() == 0) {
		  System.out.println("Join target is empty!");
		  return;
		}
		cd.sendMessage(new ChatMessage.C2S_Join(target.toString()));
		ClientFrame.setEnabledChat(true);
	  }
	});
	pum.add(mi);

	// hier binden aan rt.table
	rt.getTable().addMouseListener(new MouseAdapter() {
	  public void mouseReleased(MouseEvent me) {
		maybeShowPopup(me);
	  }
	  public void mousePressed(MouseEvent me) {
		maybeShowPopup(me);
	  }
			
	  protected void maybeShowPopup(MouseEvent me) {
		if (me.isPopupTrigger()) {
		  int nr = rt.table.rowAtPoint(me.getPoint());
		  target.setLength(0);
			
		  if (nr != -1) {
			target.append(rt.getTableModel().getValueAt(nr, 0));
		  }

		  pum.show(me.getComponent(), me.getX(), me.getY());
		}
	  }
	});
		
	c.gridx = 0;
	c.gridy = 1;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = leftwindow_weightx;
	c.weighty = 1.0;
	c.insets = insets_small;
	gbl.setConstraints(rt, c);
	add(rt);
		
	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));
	final JButton b1 = createChannel = new JButton("Create Channel");
	b1.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent ae) {
		GUITab_GlobalChat.createChannel();
		ClientFrame.setEnabledChat(true);
	  }
	});

	b1.setMargin(insets_small);
	b1.setMnemonic(KeyEvent.VK_C);
	/*		JButton b2 = new JButton("Join Channel");
			b2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
		
			cd.sendMessage(new ChatMessage.C2S_Join());
			}
			});
			b2.setMargin(insets_small); */
	b1.setMnemonic(KeyEvent.VK_J);
	p1.add(b1);
	p1.add(Box.createRigidArea(new Dimension(5, 0)));
	p1.add(updateb);
	p1.add(Box.createHorizontalGlue());
	p1.add(lab2);
	p1.add(autoUpdateCheckBox);
	c.gridx = 0;
	c.gridy = 2;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.SOUTHWEST;
	c.weightx = leftwindow_weightx;
	c.weighty = 0.0;
	c.insets = insets_small;
	gbl.setConstraints(p1, c);
	add(p1);

  }

  /**
   * Messagehandler called by ClientDaemon.
   * @param m The ChannelMessage.
   * @see quest.global.chat.ChannelMessage
   * @exception RuntimeException when msg is not a ChannelMessage.
   */
  public void incomingMessage(Message m) {
	if (!(m instanceof ChannelMessage)) {
	  throw new RuntimeException("expected ChannelMessage");
	}

	ChannelMessage cm = (ChannelMessage) m;
	if (cm instanceof ChannelMessage.S2C_FullUpdate) {
	  ChannelMessage.S2C_FullUpdate fu = (ChannelMessage.S2C_FullUpdate) cm;
			
	  channelData.setSize(0); // first clear all
	  for (int i = 0; i < fu.array.length; i++) {
		if (fu.array[i] == null) {
		  System.out.println("null ignored in fullupdate at position " + i);
		  continue;
		}

		Vector vec = new Vector(columnNames.length);
		vec.addElement(fu.array[i].name);
		vec.addElement(fu.array[i].operator);
		vec.addElement(ChatMessage.channelModes[fu.array[i].state]);
		vec.addElement(new Integer(fu.array[i].players));
		vec.addElement(new Integer(fu.array[i].spectators));
		channelData.addElement(vec);
	  }
	  rt.getTableModel().fireTableDataChanged();
	} else if (cm instanceof ChannelMessage.S2C_PartialUpdate) {
	  ChannelMessage.S2C_PartialUpdate pu = (ChannelMessage.S2C_PartialUpdate) cm;
			
	  ChannelInfo chan;
	  int idx;
			
	  for (int i = 0; i < pu.updateArray.length; i++) {
		switch (pu.updateArray[i].mode) {
		case ChannelMessage.S2C_PartialUpdate.UPDATE_MODIFY:
		  chan = pu.updateArray[i].channel;
					
		  idx = getChannelIndex(chan.name);
		  if (idx == -1) {
			System.out.println("invalid index retrieved on ChannelMessage.S2C_PartialUpdate - modify");
			return;
		  }
		  channelData.setElementAt(pu.updateArray[i].channel, idx);
		  rt.getTableModel().fireTableRowsUpdated(idx, idx);
		  break;
		case ChannelMessage.S2C_PartialUpdate.UPDATE_DELETE:
		  chan = pu.updateArray[i].channel;
					
		  idx = getChannelIndex(chan.name);
		  if (idx == -1) {
			System.out.println("invalid index retrieved on ChannelMessage.S2C_PartialUpdate - delete");
			return;
		  }
		  channelData.removeElementAt(idx);
		  rt.getTableModel().fireTableRowsDeleted(idx, idx);
		  break;
		case ChannelMessage.S2C_PartialUpdate.UPDATE_INSERT:
		  channelData.addElement(pu.updateArray[i].channel);
		  rt.getTableModel().fireTableRowsInserted(channelData.size()-1, channelData.size()-1);
		  break;
		default:
		  System.out.println("invalid operation on ChannelMessage.S2C_PartialUpdate");
		  return;
		}
	  }
	}
  }
	

  // name is name of the channel which servers as key in the table
  /**
   * Gets the index of the channel in the table.
   * @param name The String that represents the channel.
   * @return the index of the channel or -1 when not found.
   */
  private int getChannelIndex(String name) {
	for (int i = 0; i < channelData.size(); i++) {
	  ChannelInfo ch = (ChannelInfo) channelData.elementAt(i);
	  if (ch.name.equals(name)) {
		return i;
	  }
	}
	return -1;
  }

}
