package quest.client.chat;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;
import quest.client.*;
import quest.client.net.*;
import quest.global.chat.*;

/**
 * Represents the 'global' chat frame. Here users can chat and create a channel in order to start a game.
 *
 * @author Sander Brandenburg
 */
public class GUITab_GlobalChat extends GUITab_Chat implements PopupMenuListener {

    /**
     * The 'Create Channel' button.
     */
  protected JButton createChannel;


    /**
     * Empty sole constructor.
     */
  public GUITab_GlobalChat() {
  }
	
    /**
     * Initializes the globalchat GUI.
     */
  // we don't have to override the packetHandler
  protected void initForm() {
	GridBagLayout gbl = new GridBagLayout();
	setLayout(gbl);
		
	GridBagConstraints c = new GridBagConstraints();

	chatLabel = new JLabel(""); // dummy - need not change for now
	JLabel lab1 = new JLabel("You are now in the Global channel under the name of " + ClientFrame.getUsername());
	lab1.setDisplayedMnemonic(KeyEvent.VK_G);
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.SOUTHWEST;
	c.weightx = leftwindow_weightx;
	c.weighty = 0.0;
	c.insets = insets_small;
	gbl.setConstraints(lab1, c);
	add(lab1);
		
	// 'users' label
	JLabel lab2 = new JLabel("Users");
	lab2.setDisplayedMnemonic(KeyEvent.VK_U);
	c.gridx = 1;
	c.gridy = 0;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.SOUTHWEST;
	c.weightx = rightwindow_weightx;
	c.weighty = 0.0;
	c.insets = insets_small;
	gbl.setConstraints(lab2, c);
	add(lab2);
		
	// the chat textarea for chattext
	JTextPane tp1 = textPane = new JTextPane();
	tp1.setEditable(false);
	textDoc = tp1.getDocument();
	initStyles();
		
	JScrollPane sp1 = new JScrollPane(tp1, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	c.gridx = 0;
	c.gridy = 1;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = leftwindow_weightx;
	c.weighty = 1.0;
	c.insets = insets_small;
	c.gridheight = 2;
	gbl.setConstraints(sp1, c);
	c.gridheight = 1;
	add(sp1);
		  

	// the line for entering your own chat
	JTextField tf1 = textField = new JTextField();
	lab1.setLabelFor(tf1);
	tf1.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent ae) {
		if (whispermode)
		  sendWhisper();
		else
		  sendChat();
	  }
	});
	whisperLabel = new JLabel();
	textFieldPanel = new JPanel();
	textFieldPanel.setLayout(new BoxLayout(textFieldPanel, BoxLayout.X_AXIS));
	textFieldPanel.add(textField);
	c.gridx = 0;
	c.gridy = 3;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = leftwindow_weightx;
	c.weighty = 0.0;
	c.insets = insets_small;
	c.gridwidth = 2;
	gbl.setConstraints(textFieldPanel, c);
	c.gridwidth = 1;
	add(textFieldPanel);
		
	RankingTable rt = spectatorTable = playerTable = new RankingTable("retrieving users");
	final String[] columnNames = {"Users"};
	final Class[] columnTypes = {String.class};
	spectators = players = new Vector();
	VectorTableModel tm = new VectorTableModel(columnNames, columnTypes, players);
		
	rt.setTableMode(tm, true);
	rt.table.getTableHeader().setResizingAllowed(false);
	rt.table.setShowGrid(false);
	lab2.setLabelFor(rt);
	c.gridx = 1;
	c.gridy = 1;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = rightwindow_weightx;
	c.weighty = 1.0;
	c.insets = insets_small;
	gbl.setConstraints(rt, c);
	add(rt);
		
	// the administrator buttons and whisper mode
	JPanel jp2 = new JPanel();
	jp2.setLayout(new BoxLayout(jp2, BoxLayout.X_AXIS));
	final JButton b3 = createChannel = new JButton("Create Channel");
	b3.setMnemonic(KeyEvent.VK_C);
	b3.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent ae) {
		createChannel();
		ClientFrame.setEnabledChat(true);
	  }
	});
	b3.setAlignmentX(Component.CENTER_ALIGNMENT);
	jp2.add(b3);

	JButton b4 = new JButton("Quit");
	b4.setMnemonic(KeyEvent.VK_Q);
	b4.setAlignmentX(Component.CENTER_ALIGNMENT);
	b4.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent ae) {
		ClientFrame.quit();
	  }
	});
	jp2.add(b4);

		
	if (usermode == ChatMessage.USERMODE_PLAYERADMIN) {
	  JButton b5 = new JButton("Kick");
	  b5.setMnemonic(KeyEvent.VK_K);
	  b5.setAlignmentX(Component.CENTER_ALIGNMENT);
	  b5.setMargin(insets_small);
	  jp2.add(b5);
	}

	c.gridx = 1;
	c.gridy = 2;
	c.fill = GridBagConstraints.VERTICAL;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = rightwindow_weightx;
	c.weighty = 0.0;
	c.insets = insets_small;
	gbl.setConstraints(jp2, c);
	add(jp2);
			
  }	
	
    /**
     * This method is overrided from GUITab_Chat to always enable this frame.
     */
  public void setEnableWindow(boolean flag) {
	// blank intentionaly
  }

    /**
     * Initializes popup for the globalchat user table.
     */
  protected void initPopup() {
	if (playerPopup != null) {
	  playerTable.table.removeMouseListener(playerPopupAdapter);
	}
		
	playerPopup = new JPopupMenu();
	JPanel jpa = new JPanel();
	jpa.setLayout(new BoxLayout(jpa, BoxLayout.X_AXIS));
	jpa.add(Box.createHorizontalGlue());
	jpa.add(new JLabel("User Action"));
	jpa.add(Box.createHorizontalGlue());
	playerPopup.add(jpa);
	registerMenuItem(playerPopup, "Whisper", "whisper", KeyEvent.VK_W);
	if (usermode == ChatMessage.USERMODE_PLAYERADMIN)
	  registerMenuItem(playerPopup, "Kick", "kick", KeyEvent.VK_K);
		
	playerPopupAdapter = new PopupMenuMouseListener(playerPopup, playerTable);
	playerTable.table.addMouseListener(playerPopupAdapter);
		
  }

    /**
     * Toggles whether the 'create channel' button is enabled or not.
     * @param flag Toggles whether the create button is enabled or not.
     */
  public void setEnabledCreateChannel(boolean flag) {
	createChannel.setEnabled(flag);
  }
	

  private static String getPostfix(String name) {
	if (name == null)
	  return null;
		
	char c = name.charAt(name.length()-1);
	if (c == 's' || c == 'x')
	  return "'";
	else
	  return "'s";
		
  }

    /**
     * Tries to gracefully shut down the channel. Blocks until a logout message has been received.
     */
  public void quit() {
	// assumed thread is killed when response takes too long
	cd.sendMessage(new ChatMessage.C2S_Logout(ChatMessage.GLOBALCHANNELNAME));
	while (loggedin) {
	  try {
		Thread.currentThread().sleep(10);
	  } catch (InterruptedException ie) {} // ignore
	}
  }

    /**
     * Creates a private channel for the client.
     */
  public static void createChannel() {
	ChatMessage.C2S_Create cr = new ChatMessage.C2S_Create(ClientFrame.getUsername() + getPostfix(ClientFrame.getUsername()) + " game");
	ClientDaemon.getInstance().sendMessage(cr);
  }
		
  private class PopupMenuMouseListener extends MouseAdapter {
	JPopupMenu	pum;
	RankingTable		rt;
		
	PopupMenuMouseListener(JPopupMenu pum, RankingTable rt) {
	  this.pum = pum;
	  this.rt = rt;
	}
		
	public void mousePressed(MouseEvent e) {
	  int nr = rt.table.rowAtPoint(e.getPoint());
	  if (nr == -1) {
		tableTarget = null;
	  } else {
		tableTarget = (String) rt.tm.getValueAt(nr, 0);
	  }
			
	  if (rt.table != null) {
		maybeShowPopup(e);
	  }
	}

	public void mouseReleased(MouseEvent e) {
	  if (rt.table != null)
		maybeShowPopup(e);
	}

	public void mouseExited(MouseEvent e) {
	  rt.table.clearSelection();
	}
		
	private void maybeShowPopup(MouseEvent e) {
	  if (rt.table != null && e.isPopupTrigger()) {
		int x = rt.table.rowAtPoint(e.getPoint());
				
		if (x != -1) { // && y != -1) {
		  rt.table.clearSelection();
		  rt.table.setRowSelectionInterval(x, x);
		  rt.table.requestFocus();
		}
				
		pum.show(e.getComponent(),
				 e.getX(), e.getY());
	  }
	}
  }
	
  //POPUPMENULISTENER
    /**
     * Empty.
     */
  public void popupMenuCanceled(PopupMenuEvent e) {
  }
         
    /**
     * Clears selection of the playerTable.
     */
  public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
	if (e.getSource() == playerPopup && playerTable.table != null) {
	  playerTable.table.clearSelection();
	}
  }

    /**
     * Empty.
     */
  public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
		
  }
	
}
