package quest.client.chat;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.*;
import javax.swing.text.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.awt.*;

import quest.client.ClientFrame;

/**
 * Represents the Help frame in the Chat GUI.
 * This class also contains a JEditorPane and loads the help.html file from the codebase.
 * 
 * @author Sander Brandenburg
 */
public class GUITab_Help extends JPanel {
    
    /**
     * The HTML screen that covers the entire window.
     */
    protected JEditorPane html;
	
    /**
     * Initializes JEditorPane and opens help.html from the codebase given by ClientFrame.
     */
    public GUITab_Help() {
	try {
	    URL url = new URL(ClientFrame.getCodeBase(), "help.html");
			
	    html = new JEditorPane(url);
	    html.setEditable(false);
	    html.addHyperlinkListener(createHyperLinkListener());
				
	    JScrollPane scroller = new JScrollPane();
	    JViewport vp = scroller.getViewport();
	    vp.add(html);
	    setLayout(new GridLayout(1, 1));
	    add(scroller);

	} catch (MalformedURLException e) {
	    System.err.println("Couldn't load helpfile - Malformed URL: " + e);
	} catch (IOException e) {
	    System.err.println("IOException while loading helpfile: " + e);
	}
    }
    
    /**
     * Gets a simple HyperlinkListener that updates the page according to its HyperlinkEvent.
     * @return a simple HyperlinkListener that updates the page according to its HyperlinkEvent.
     */
    protected HyperlinkListener createHyperLinkListener() {
	return new HyperlinkListener() {
		public void hyperlinkUpdate(HyperlinkEvent e) {
		    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
			if (e instanceof HTMLFrameHyperlinkEvent) {
			    ((HTMLDocument)html.getDocument()).processHTMLFrameHyperlinkEvent((HTMLFrameHyperlinkEvent)e);
			} else {
			    try {
				html.setPage(e.getURL());
			    } catch (IOException ioe) {
				System.out.println("Couldn't load helpfile: " + ioe);
			    }
			}
		    }
		}
	    };
    }

}
