package quest.client.chat;

import java.awt.*;
import java.awt.event.*;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.*;
import javax.swing.JTable;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.table.*;
import quest.client.ClientFrame;
import quest.client.net.*;
import quest.global.chat.*;
import quest.global.net.*;

/**
 * Represents the search panel in the Chat GUI.
 * Registers SearchMessage to the ClientDaemon.
 *
 * @author Sander Brandenburg
 * @see quest.global.chat.SearchMessage
 */
public class GUITab_Search extends JPanel implements MessageClientListener {

    /**
     * The popupmenu that is shown when an entry in the resultwindow is activated.
     */
    protected JPopupMenu popup;
    /**
     * The searchbutton.
     */ 
    protected JButton searchButton;
    /**
     * The resultwindow.
     */
    protected RankingTable rt;
    /**
     * The search-query texfield
     */
    protected JTextField textField;

    /**
     * The object communicator between client and server.
     */
    protected ClientDaemon cd = ClientDaemon.getInstance();
    
    /**
     * Initializes the form and registers SearchMessage on the ClientDaemon.
     */
    public GUITab_Search() {
	super();
		
	initForm();
		
	cd.registerProces(SearchMessage.class, this);
    }
    
    private final double leftwindow_weightx = 1.0;
    private final Insets insets_none = new Insets(0, 0, 0, 0);
    private final Insets insets_small = new Insets(2, 2, 2, 2);		
	
    /**
     * Initializes the searchform.
     */ 
    protected void initForm() {
	GridBagLayout gbl = new GridBagLayout();
	setLayout(gbl);
		
	GridBagConstraints c = new GridBagConstraints();
		
	JLabel lab1 = new JLabel("Username");
	lab1.setDisplayedMnemonic(KeyEvent.VK_U);
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.SOUTHWEST;
	c.weightx = leftwindow_weightx;
	c.weighty = 0.0;
	c.insets = new Insets(2, 5, 2, 2);
	gbl.setConstraints(lab1, c);
	add(lab1);
		
	JTextField tf1 = textField = new JTextField(30);
	tf1.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    doSearch(textField.getText());
		}
	    });
	JButton b1 = searchButton = new JButton("Search");
	b1.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    doSearch(textField.getText());
		}
	    });
	b1.setMnemonic(KeyEvent.VK_S);
	JPanel p1 = new JPanel();
	p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));
	p1.add(tf1);
	p1.add(Box.createRigidArea(new Dimension(5, 0)));
	p1.add(Box.createHorizontalGlue());
	p1.add(b1);
	c.gridx = 0;
	c.gridy = 1;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = leftwindow_weightx;
	c.weighty = 0.0;
	c.insets = insets_small;
	gbl.setConstraints(p1, c);
	add(p1);
		
	rt = new RankingTable("Enter Search Query");
	TitledBorder title1 = BorderFactory.createTitledBorder("Results");
	rt.setBorder(title1);
	c.gridx = 0;
	c.gridy = 2;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = leftwindow_weightx;
	c.weighty = 1.0;
	c.insets = new Insets(5, 5, 5, 5);
	gbl.setConstraints(rt, c);
	add(rt);
	
	final StringBuffer target = new StringBuffer(40);
	//register listeners
	popup = new JPopupMenu();
	JMenuItem menuItem = new JMenuItem("Join game");
	menuItem.setMnemonic(KeyEvent.VK_J);
	menuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    cd.sendMessage(new ChatMessage.C2S_Join(target.toString()));
		}
	    });
	popup.add(menuItem);
		
	//Add listener to components that can bring up popup menus.
	MouseListener popupListener = new MouseAdapter() {
		public void mousePressed(MouseEvent e) {
		    maybeShowPopup(e);
		}
		
		public void mouseReleased(MouseEvent e) {
		    maybeShowPopup(e);
		}
		
		private void maybeShowPopup(MouseEvent e) {
		    if (e.isPopupTrigger()) {
			int nr = rt.table.rowAtPoint(e.getPoint());
			target.setLength(0);
			
			if (nr != -1) {
			    target.append(rt.getTableModel().getValueAt(nr, 0));
			}
			popup.show(e.getComponent(),
				   e.getX(), e.getY());
		    }
		}
	    };
	rt.addMouseListener(popupListener);

    }

    /**
     * Checks if str is not null or 0 length and sends C2S_Search message to server.
     */
    protected void doSearch(String str) {
	if (str == null || str.length() == 0)
	    return;
      
	SearchMessage.C2S_Search sr = new SearchMessage.C2S_Search(str);
	cd.sendMessage(sr);		
    }

  /**
   * Messagehandler called by ClientDaemon.
   * @param m The SearchMessage.
   * @see quest.global.chat.SearchMessage
   * @exception RuntimeException when msg is not a SearchMessage.
   */
    public void incomingMessage(Message msg) {
	  if (!(msg instanceof SearchMessage)) {
		throw new RuntimeException("expected SearchMessage");
	  }
	SearchMessage sm = (SearchMessage) msg;
		
	if (sm instanceof SearchMessage.S2C_Result) {
	    SearchMessage.S2C_Result res = (SearchMessage.S2C_Result) sm;
			
	    if (res.result == null) {
		rt.setLabelMode("empty result");
		return;
	    }
			
	    initTable();
	    rt.getTableModel().getData().removeAllElements();
	    rt.getTableModel().getData().ensureCapacity(res.result.length);
			
	    for (int i = 0; i < res.result.length; i++) {
		Vector vec = new Vector(6);
		SearchMessage.S2C_Result.SearchResult sr = res.result[i];

		vec.add(sr.username);
		vec.add(ChatMessage.userModes[sr.userstate]);
		vec.add(new Integer(sr.rating));
		vec.add(new Integer(sr.nrgames));
		vec.add(sr.currentchannel);
		vec.add(ChatMessage.channelModes[sr.gamestate]);
		rt.getTableModel().getData().add(vec);
	    }
	    rt.getTableModel().fireTableDataChanged();
	}
    }
    
    /**
     * Initializes a new resultwindow
     */
    protected void initTable() {
	final String[] columnNames = {
	    "User", "state", "Rating", "#Games", "Current channel", "Gamestate"
	};
	final Class[] classTypes = {
	    String.class, String.class, Integer.class, Integer.class, String.class, String.class
	};
		
	Vector data = new Vector();
	VectorTableModel vtm = new VectorTableModel(columnNames, classTypes, data);
	rt.setTableMode(vtm, true);
    }
}
