package quest.client.chat;

import java.util.Vector;
import javax.swing.table.AbstractTableModel;

/**
 * Implements AbstractTableModel for use with RankingTable. It relies on a Vector used for data representation.
 * Each element in this Vector is another Vector that represents a row. Each latter Vector has to have the values
 * according to the given classTypes array on construction.
 *
 * @see RankingTable
 * @author Sander Brandenburg
 */
public class VectorTableModel extends AbstractTableModel {
    private String[] columnNames;
    private Vector data = new Vector(100);
    private Class[] classTypes;
   
    /**
     * Constructs VectorTableModel.
     * @param columnNames The array of Strings for the columns that has to be drawn in the table header.
     * @param classTypes The array of Class types of the cell contents to aid the CellRenderers.
     * @param data The Vector that represents the data. Each element is another Vector that represents a row in the model of the table.
     * Each element in the latter Vector should be of the type according to classTypes.
     * @exception RuntimeException when columnames.length != classTypes.length, or when the data argument constraint described above is violated.
     */
    public VectorTableModel(String[] columnNames, Class[] classTypes, Vector data) {
	if (columnNames.length != classTypes.length)
	    throw new RuntimeException("columnNames length is not equal to classTypes");
		
	if (columnNames == null || classTypes == null || data == null)
	    throw new RuntimeException("VectorTableModel arguments can't be null");

	for (int i = 0; i < data.size(); i++) {
	    Object  o = data.elementAt(i);
	    if (o instanceof Vector) {
		Vector vec = (Vector) o;
		if (vec.size() != columnNames.length) {
		    throw new RuntimeException("element " + i + " of data doesn't have equal length(" + vec.size() + ") to columnNames(" + columnNames.length);
		}
		for (int j = 0; j < vec.size(); j++) {
		    if (vec.elementAt(j).getClass() != classTypes[j]) {
			throw new RuntimeException("element " + i + " of data doesn't have equal length(" + vec.size() + ") to columnNames(" + columnNames.length);
		    }
		}
	    } else {
		throw new RuntimeException("element " + i + " is not a Vector");
	    }
	}
	this.data = data;
	this.columnNames = columnNames;
	this.classTypes = classTypes;
    }

    /**
     * @return data Vector
     */
    public Vector getData() {
	return data;
    }
    public int getColumnCount() {
        return columnNames.length;
    }
	
    public int getRowCount() {
        return data.size();
    }
	
    public String getColumnName(int col) {
        return columnNames[col];
    }

    public Object getValueAt(int row, int col) {
	Vector vec = (Vector) data.elementAt(row);
	return vec.elementAt(col);
    }
	
    public Class getColumnClass(int column) {
	return classTypes[column];
    }
}
