package quest.client.game;

import java.awt.*;
import javax.swing.Icon;

/**
 * An icon that shows a round circle of a certain color.
 *
 * @author Mathijs den Burger
 */ 

class ColorIcon implements Icon {

	private static final int INSETS = 1;		// pixels
		
	private int width, height;
	private Color color;

	/**
	 * Constructs an icon of the specified color and size.
	 * @param color The color of the icon
	 * @param width The width of the icon (in pixels)
	 * @param height The height of the icon (in pixels)
	 */
	ColorIcon(Color color, int width, int height) {
		this.color = color;
		this.width = width;
		this.height = height;
	}
	
	/**
	 * Gives the width of the icon (in pixels)
	 * @return The width of the icon
	 */
	public int getIconWidth() {
		return width;
	}
	
	/**
	 * Gives the height of the icon (in pixels)
	 * @return The height of the icon
	 */
	public int getIconHeight() {
		return height;
	}
	
	/**
	 * Paints the icon on the given graphics at the specified location.
	 * @param c The component in which the icon is drawn. This can be null
	 * @param g The graphics object to draw the icon in
	 * @param x The x-coordinate in the graphics object of the upper-left corner of the icon
	 * @param y The y-coordinate in the graphics object of the upper-left corner of the icon
	 */
	public void paintIcon(Component c, Graphics g, int x, int y) {
		Color tempColor = g.getColor();
		g.setColor(color);
		g.fillOval(x + INSETS, y + INSETS, width - INSETS - 1, height - INSETS - 1);
		g.setColor(tempColor);
	}
	
}