package quest.client.game;

import java.awt.Component;
import java.awt.event.MouseEvent;

/**
 * An event generated when a user picks an object in the 3D scene.
 * These events are generated by a {@link quest.client.game.PickBehavior}.
 *
 * @author Mathijs den Burger
 */

class PickEvent extends MouseEvent {

	/**
	 * The object that was picked
	 */
	PickableObject picked;

	/**
	 * Constructs a pick event. For a description of the other parameters,
	 * see {@link java.awt.event.MouseEvent}.
	 * @param picked The object that was picked
	 */
	public PickEvent(Component source, PickableObject picked, int id,
		long when, int modifiers, int x, int y, int clickCount, boolean isPopupTrigger) {
		super(source, id, when, modifiers, x, y, clickCount, isPopupTrigger);
		this.picked = picked;
	}

	/**
	* Gives the object that was picked
	 * @return The object that was picked
	 */
	public PickableObject getPicked() {
		return picked;
	}

}