package quest.client.game;

import quest.client.ClientFrame;
import java.net.URL;

/**
 * <P>
 * Keeps all the global runtime settings of a game. This class is a Singleton,
 * so all other components can access these settings easily.</P>
 * <P>
 * This class is a <A HREF="../../../overview-summary.html#patternSingleton">Singleton</A>.</P>
 *
 * @author Mathijs den Burger
 */

class Settings {

	private static Settings uniqueInstance = null;
	private URL codeBase;

	/**
	 * Constructs the Settings object. You can get a reference to this object
	 * with the method {@link quest.client.game.Settings#instance()}.
	 */
	protected Settings() {
	    codeBase = ClientFrame.getCodeBase();
	}

	/**
	 * Gives a reference to the single Settings class.
	 * @return A reference to the single Settings class.
	 */
	public static Settings instance() {
		if (uniqueInstance == null)
			uniqueInstance = new Settings();
		return uniqueInstance;
	}

	/**
	 * Sets the code base of the program manually.
	 * @param codeBase The code base of the program
	 */
	public void setCodeBase(URL codeBase) {
		this.codeBase = codeBase;
	}

	/**
	 * Gives the code base of the program
	 * @return The code base of the program
	 */
	public URL getCodeBase() {
		return codeBase;
	}

}
