package quest.client.game;

import quest.global.game.GameSpectator;
import java.awt.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;

/**
 * Displays a list of spectator's names.
 *
 * @author Mathijs den Burger
 */

class SpectatorList extends JComponent {

	private static final Font SPECTATOR_FONT = new Font("SansSerif", Font.PLAIN, 12);
	private static final EmptyBorder SPECTATOR_BORDER = new EmptyBorder(0, 2, 0, 0);
	private static final Color SPECTATOR_FOREGROUND = Color.darkGray,
		LIST_BACKGROUND = Color.lightGray,
		HEADER_FOREGROUND = new Color(0x33, 0x33, 0x44);

	private static final String HEADER_TEXT = " Spectators:";

    private JList list;

    /**
     * Constructs a list that contains the names of all the given
     * spectators. The names are displayed in the order they are contained
	 * in the given vector.
     * @param spectators A vector of {@link quest.global.game.GameSpectator}
     * objects. Subsequent changes to the givn vector aren't reflected
     * in the list, you should use {@link quest.client.game.SpectatorList#setData(java.util.Vector)}
     * for that.
     */
    SpectatorList(Vector spectators) {
		// build the list
		list = new JList(spectators);
		list.setCellRenderer(new GameSpectatorListCellRenderer());
		list.setEnabled(false);
		list.setBackground(LIST_BACKGROUND);

		// place the list in a scrollpane
		JScrollPane scrollpane = new JScrollPane(list);

		// build header to put on top of the list
	    JLabel header = new JLabel(HEADER_TEXT, SwingConstants.LEADING);
	    header.setForeground(HEADER_FOREGROUND);

	    // set layout
		GridBagLayout gridbag = new GridBagLayout();
       	GridBagConstraints c = new GridBagConstraints();
		setLayout(gridbag);

       	// add the header
       	c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridheight = 1;
        c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		c.weighty = 0.0;
		c.ipadx = 0;
		c.ipady = 0;
        c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(header, c);
        add(header);

		// add scrollpane
		c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridheight = GridBagConstraints.REMAINDER;
        c.gridy = GridBagConstraints.RELATIVE;
        c.fill = GridBagConstraints.BOTH;
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.ipadx = 0;
		c.ipady = 0;
		c.anchor = GridBagConstraints.CENTER;
		gridbag.setConstraints(scrollpane, c);
        add(scrollpane);
	}

	/**
	 * Removes the old contents of the list and displays the names
	 * of the new given spectators.
	 * @param spectators The spectators to display the names of.
	 * The names are displayed in the order they are contained
	 * in the vector.
	 */
    public void setData(Vector spectators) {
    	list.setListData(spectators);
    }

	private class GameSpectatorListCellRenderer extends JLabel implements ListCellRenderer {

		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            if (value instanceof GameSpectator) {
            	GameSpectator spectator = (GameSpectator)value;
				setBorder(SPECTATOR_BORDER);
				setFont(SPECTATOR_FONT);
	            setText(spectator.getName());
            	setForeground(SPECTATOR_FOREGROUND);
            } else {
            	setText("");
            }
            return this;
        }

    }

}


