package quest.client.game;

import quest.global.game.*;
import javax.swing.*;
import javax.swing.border.BevelBorder;
import java.awt.*;

/**
 * Displays a rectangular area with some text in it. The text to display can 
 * be changed.
 *
 * @author Mathijs den Burger
 */

class StatusBar extends JComponent {

	private static final String DEFAULT_FONT_NAME = "SansSerif";
	private static final int DEFAULT_FONT_STYLE = Font.PLAIN;
	private static final int DEFAULT_FONT_SIZE = 10;
	private static final int NR_ROWS = 2, NR_COLUMNS = 30;
	private static final Color DEFAULT_FOREGROUND = new Color(0x33, 0x33, 0x44),
		STATUS_BAR_BACKGROUND = Color.lightGray;

	private JTextArea statusText;

	/**
	 * Constructs a status bar with no text in it.
	 */
	StatusBar() {
		// build status text
		statusText = new JTextArea();
		//statusText.setRows(NR_ROWS);
		statusText.setFont(new Font(DEFAULT_FONT_NAME, DEFAULT_FONT_STYLE, DEFAULT_FONT_SIZE));
		statusText.setRows(NR_ROWS);
		statusText.setColumns(NR_COLUMNS);
		statusText.setEditable(false);
		statusText.setDisabledTextColor(DEFAULT_FOREGROUND);
		statusText.setLineWrap(true);
		statusText.setWrapStyleWord(true);
		statusText.setText("HOEBA");
		statusText.setBackground(STATUS_BAR_BACKGROUND);
		statusText.setBorder(BorderFactory.createEmptyBorder());
		
		// set layout
		GridBagLayout gridbag = new GridBagLayout();
       	GridBagConstraints c = new GridBagConstraints();
		setLayout(gridbag);
		
		// add status text
		c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridheight = GridBagConstraints.REMAINDER;
        c.gridy = GridBagConstraints.RELATIVE;
        c.fill = GridBagConstraints.BOTH;
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.ipadx = 0;
		c.ipady = 0;
		c.anchor = GridBagConstraints.CENTER;
		gridbag.setConstraints(statusText, c);
        add(statusText);
		
		setBackground(STATUS_BAR_BACKGROUND);
		setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
	}
	
	/**
	 * Displays the given text in the status bar.
	 * @param text The text to display
	 */
	public void setText(String text) {
		statusText.setText(text);
	}
	
}
