package quest.client.game;

import quest.global.game.MazeCorridor3D;
import quest.global.game.MazePosition3D;
import quest.global.game.Selectable;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.picking.PickTool;

/**
 * Represents a 3D teleport in the maze. A teleport is merely a round platform
 * that is stuck to the ground of a corridor.
 *
 * @author Mathijs den Burger
 */
 
class Teleport extends PickableQuestObject {

	private static final float HEIGHT = 1f;
	
	private static final float RADIUS = Walls.CORRIDOR_WIDTH * 0.333f;

	private MazeCorridor3D environment;

	/**
	 * Constructs a teleport that lies (logically) in the given maze corridor.
	 * @param environment The corridor in which the teleport is (logically)
	 */
	Teleport(MazeCorridor3D environment) {
		super(environment);
		this.environment = environment;
	}
	
	/**
	 * Gives the corridor in which the teleport is (logically).
	 * @return The corridor
	 */
	public MazeCorridor3D getEnvironment() {
		return environment;
	}

	/**
	 * Builds the 3D model of the teleport.
	 * @return The root node of the 3D model
	 */
	public Node buildSelectableModel() {
		NodeComponentFactory ncf = NodeComponentFactory.instance();

		Appearance appearance = ncf.getAppearance(ncf.getMaterial(new Color3f(0.4f, 0.4f, 0.4f), new Color3f(0.4f, 0.4f, 0.4f), new Color3f(0.4f, 0.4f, 0.4f), new Color3f(0.1f, 0.1f, 0.1f), 128f), null);
		int primFlags = Primitive.GENERATE_NORMALS | Primitive.ENABLE_GEOMETRY_PICKING;
		Cylinder teleport = new Cylinder(RADIUS, HEIGHT, primFlags, appearance);

		// position teleport object in the middle of an imaginary corridor block
		Transform3D internalTrans = new Transform3D();
		internalTrans.setTranslation(new Vector3f(CorridorBlock.BLOCK_SIZE / 2, Floor.HEIGHT, CorridorBlock.BLOCK_SIZE / 2));
		TransformGroup internalTg = new TransformGroup(internalTrans);
		internalTg.addChild(teleport);

		return internalTg;
	}

}