package quest.client.net;

import java.net.*;

/**
 * Stores the configuration of the clients.
 * Used as a central point for all configuration settings.
 *
 * @author    Merijn Evertse
 */
class ClientConfiguration {
    private InetAddress serverAddress;
    private int serverPort;
    private int sleeptime;  // in milliseconds
    private static ClientConfiguration uniqueInstance = null;

    protected ClientConfiguration() {
	try {
	    serverAddress = InetAddress.getByName("localhost");
	} catch (UnknownHostException e) {
	    System.out.println(e.getMessage());
	}
	serverPort  = 5000;
	sleeptime   = 1;
    }
    
    /**
     * Returns the internet address of the server.
     *
     * @return    The InetAddress of the server
     */
    public InetAddress getServerAddress() {
		return serverAddress;
    }

	/**
	 * Returns the port number the server is listening on.
	 *
	 * @return    The server port number
	 */
    public int getServerPort() {
		return serverPort;
    }
  
  	/**
  	 * Returns the time (in milliseconds) the client should wait between communication
  	 * actions, to prevent the communication to use too much cpu time.
  	 *
  	 * @return    The sleep time in milliseconds
  	 */
    public int getSleeptime() {
  		return sleeptime;
    }
    
    /**
     * Returns an instance of this class, making sure only one object of it exists.
     *
     * @return    A reference to the instance of this class
     */
    public static ClientConfiguration getInstance() {
	if (uniqueInstance == null) {
	    uniqueInstance = new ClientConfiguration();
	}
	return uniqueInstance;
    }
}
