package quest.client.net;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import quest.global.net.*;

/**
 * Client side message receiving thread for handling incoming messages and transporting it
 * to the right registered proces.
 * For a proces to be able to use the message stream, it has to register.
 *
 * @author    Merijn Evertse
 */
public class MessageClientThread extends Thread {
  private MessageStream messageStream;
  private Vector processes;
  private ClientConfiguration configuration;
  private boolean allowedToRun;

  /**
   * Initializes the thread by binding to the given message stream for listening.
   *
   * @param    messageStream    The message stream to listen on for incoming messages
   */
  public MessageClientThread(MessageStream messageStream) {
    this.messageStream = messageStream;
    processes = new Vector();
    configuration = ClientConfiguration.getInstance();
	allowedToRun = true;
  }

  /**
   * Registers a proces consisting of a message type and a method for handling
   * the message. After the registration of the proces all received messages
   * intended for the registerd proces are supplied to the registerd message
   * handler.
   *
   * @param    proces        The proces to register
   */
  public void registerProces(Proces proces) {
    processes.addElement(proces);
    
    System.out.println("Registerred proces with " + 
		       proces.messageType.getName() +
		       ", " + proces.messageListener.getClass().getName()); //DEBUG
  }

	/**
	 * Unregisters a proces. The proces will now no longer receive incoming messages.
	 * NOTE: If incoming messages are of a type only this proces could handle, they will
	 * from now on be discarded.
	 *
	 * @param    proces    The proces to unregister
	 */
  	public void unregisterProces(Proces proces) {
		processes.removeElement(proces);
  	}

  private void handleMessage(Message message) {
    boolean messageHandled = false;

    for (int i = 0; i < processes.size(); i++) {
      if (((Proces) processes.elementAt(i)).messageType.isInstance(message)) {
		// the message is for proces 'i' so give it to this proces' handler
		MessageClientListener messageListener = (MessageClientListener)
			((Proces) processes.elementAt(i)).messageListener;

		try {
		messageListener.incomingMessage(message);
		} catch (Exception e) {
			e.printStackTrace();
			//DESS wil toch wel weten wat die exception is
		  //System.out.println("Exception occured in proces' messagehandler");
		}

		messageHandled = true;
      }
    }
    if (!messageHandled) {
      System.out.println("Couldn't find an appropriate proces to handle the message");
      // message is discarded
    }
  }

  /**
   * Runs this thread.
   */
  public void run() {
    while (allowedToRun) {
	  handleMessage(messageStream.receiveMessage());
    }
  }
  
	/**
	 * Closes the listener.
	 */
  	public void close() {
		allowedToRun = false;
  	}
}
