package quest.global.chat;

import java.io.*;

/**
 * Contains all the information about a channel. Used as a container class for sending clients
 * channel information.
 *
 * @author    Merijn Evertse
 */
public class ChannelInfo implements Serializable {
	
	/**
	 * The name of the channel.
	 */
	public String name;
	
	/**
	 * The name of the administrator of the channel.
	 */
	public String operator;
	
	/**
	 * The current status of the channel.
	 */
	public int state;

	/**
	 * Indication whether the channel is locked or unlocked.
	 */
	public boolean locked;

	/**
	 * The number of players in the channel.
	 */
	public int players;

	/**
	 * The number of spectators in the channel.
	 */
	public int spectators;
	
	/**
	 * Initializes all variables with the given parameters.
	 *
	 * @param    name        The name of the channel
	 * @param    operator    The name of the admin of the channel
	 * @param    state       The current state of the channel
	 * @param    locked      TRUE:  if the channel is locked
	 *                       FALSE: if not
	 * @param    players     The number of players in the channel
	 * @param    spectators  The number of spectators in the channel
	 */
	public ChannelInfo(String name, String operator, int state, boolean locked, int players, int spectators) {
		this.name = name;
		this.operator = operator;
		this.state = state;
		this.locked = locked;
		this.players = players;
		this.spectators = spectators;
	}
}
