package quest.global.chat;

import quest.global.net.Message;
import java.io.Serializable;

/**
 * Contains all the information about a single player entry in a ranking overview.
 * This class is used as a container class in the communication between server and client.
 *
 * @author    Merijn Evertse & Sander Brandenburg
 */
public class UserRanking implements Serializable {
	/**
	 * The current position of the player.
	 */
	public int position;
	
	/**
	 * The name of the player.
	 */
	public String username;
	
	/**
	 * The current rank of the player.
	 */
	public int ranking;
	
	/**
	 * The total number of games the player played.
	 */
	public int nrgames;
	
	/**
	 * The number of games the player won.
	 */
	public int nrwins;
	
	/**
	 * The number of games the player lost.
	 */
	public int nrlost;

	/**
	 * Initializes the object with the given parameters.
	 *
	 * @param    position    The current position of the player
	 * @param    username    The name of the player
	 * @param    ranking     The current rank of the player
	 * @param    nrgames     The total number of games the player played
	 * @param    nrwins      The number of games the player won
	 * @param    nrlost      The number of games the player lost
	 */
	public UserRanking(int position, String username, int ranking, int nrgames, int nrwins, int nrlost) {
		this.position = position;
		this.username = username;
		this.ranking = ranking;
		this.nrgames = nrgames;
		this.nrwins = nrwins;
		this.nrlost = nrlost;
	}
	
	/**
	 * Returns whether the given object is equal to this one.
	 *
	 * @return    TRUE:  if the parameters is equals to this object
	 *            FALSE: if not
	 */
	public boolean equals(Object rhs) {
		if (rhs instanceof UserRanking) {
			return username.equals(((UserRanking)rhs).username);
		} 
		return false;
	}
}
