package quest.global.game;

import java.util.Observer;
import java.util.Observable;
import quest.global.util.SerializableObservable;

/**
 * The logical representation of a spectator of an Amazing Quest game.
 * A spectator is someone who only observes the game and doesn't
 * participate in taking turns.
 *
 * @author Mathijs den Burger
 */

public class GameSpectator extends SerializableObservable implements Comparable, Cloneable {

    private String name;			// name of this spectator

	/**
	 * Creates an spectator
	 * @param name The name of the spectator
	 */
    public GameSpectator(String name) {
    	this.name = name;
    }

	/**
	 * Copy constructor used by clone()
	 */
    protected GameSpectator(GameSpectator original) {
		name = new String(original.name);
    }

	/**
	 * Gives the name of this spectator
	 * @return The name of this spectator
	 */
    public String getName() {
    	return name;
    }

    /**
     * Checks if this spectator is equal to the given one.
     * Two spectators are equal if they have the same name.
     * Names are not case sensitive.
     * @param rhs The other spectator
     * @return true if this spectator is equal to the other one, false otherwise
     */
    public boolean equals(GameSpectator rhs) {
		return (name.toLowerCase() == rhs.name.toLowerCase());
    }

	/**
	 * Compares this spectator with the given object.
	 * Comparing this spectator to anything that's not a spectator
	 * will cause a RuntimeException.
	 * @param o The object to compare this spectator with
	 */
    public int compareTo(Object o) {
		if (o instanceof GameSpectator) {
		    return name.compareToIgnoreCase(((GameSpectator)o).name);
		} else {
		    throw new RuntimeException("GameSpectator.compareTo: cannot convert object to an GameSpectator");
		}
    }

	/**
	 * Builds a copy of this object
	 * @return The copy
	 */
    public Object clone() {
		return new GameSpectator(this);
    }

}
