package quest.global.net;

import java.io.*;
import java.lang.reflect.*;

/**
 * Definition of a message for communication over the network through
 * the communication between client and server.
 * All processes running on top of the communication layer should define a
 * message that extends this one to ensure the safe transfer of their messages.
 *
 * @author    Merijn Evertse & Sander Brandenburg
 */ 
public abstract class Message implements Serializable {

	/**
	 * Returns a description of this message.
	 *	
	 * @return    The description of the message
	 */
	public String toString() {	
		return toString(this);
	}
	
	private String toString(Object obj) {
		String str = obj.getClass().getName() + "[";
		
		Field[] fields = obj.getClass().getFields();
		
		boolean first = true;

		for (int i = 0; i < fields.length; i++) {
			if ((fields[i].getModifiers() & Modifier.FINAL) != 0)
				continue;
			
			if (first) {
				first = false;
			} else {
				str += ", ";
			}

			Class c = fields[i].getType();
			
			str += fields[i].getName() + "=";
			try {
				if (c.isArray()) {
					str += "{";
					Object obj2 = fields[i].get(obj);
					
					if (obj2 == null) {
						str += "null";
					} else if (obj2 instanceof Object[]) {
						//if (!(obj2 instanceof String[]) && !(obj2 instanceof Integer[])) {
						//str += ".. zoek het maar uit .. ";
						//} else {
						{
							Object[] sa = (Object[]) obj2;
							for (int j = 0; j < sa.length; j++) {
								if (j != 0)
									str += ", ";
								str += sa[j];
								
							}
						}
					} else if (obj2 instanceof int[]) {
						int[] ia = (int[]) obj2;
						for (int j = 0; j < ia.length; j++) {
							if (j != 0)
								str += ", ";
							str += ia[j];
						}
					} else {
						str += "...unimplemented type...";
					}
					str += "}";
				} else if (c.isPrimitive()) {
					if (c == Integer.TYPE) {
						str += fields[i].getInt(obj);
					} else if (c == Boolean.TYPE) {
						str += fields[i].getBoolean(obj);
					} else {
						System.out.println("unimplemented primitive type: " + c.toString());
					}
				} else if (c == String.class) {
					String s = (String) fields[i].get(obj);
					if (s == null)
						str += "\"null\"";
					else
						str += "\"" + fields[i].get(obj) + "\"";
				} else {
					Object obj2 = fields[i].get(obj);
					str += toString(obj2);
				}
			} catch (IllegalAccessException iae) {
				str += "<unable to retrieve>";
			}
		}
		
		str += "]";
		return str;
	}	
}




