package quest.global.net;

/**
 * Defines a proces, so it can use the message communication facility.
 *
 * @author    Merijn Evertse
 */
public class Proces {

	/**
	 * The message type this proces uses.
	 */
	public Class messageType;
	
	/**
	 * The message listener this proces has for handling messages.
	 */
	public MessageListener messageListener;

	/**
	 * Provides a means for classification of processes on different types of registration.
	 */
	public int registrationType;

	/**
	 * Creates a proces with the given message type and listener for incoming messages.
	 *
	 * @param    messageType        The type of the message this proces used for communication
	 * @param    messageListener    The listener daemon of the proces for handling incoming messages
	 */
	public Proces(Class messageType, MessageListener messageListener) { 
		this.messageType = messageType;
		this.messageListener = messageListener;
		this.registrationType = 0;
	}

	/**
	 * Creates a proces with the given message type and listener for incoming messages.
	 *
	 * @param    messageType        The type of the message this proces used for communication
	 * @param    messageListener    The listener daemon of the proces for handling incoming messages
	 * @param    registrationType   The type of the proces
	 */
	public Proces(Class messageType, MessageListener messageListener, int registrationType) { 
		this.messageType = messageType;
		this.messageListener = messageListener;
		this.registrationType = registrationType;
	}

	/**
	 * Checks whether this proces is equal to the given parameter.
	 *
	 * @return    TRUE:  if both processes are equal
	 *            FALSE: if not
	 */
	public boolean equals(Proces proces) {
		return messageType.equals(proces.messageType);
	}
}

   
