package quest.global.util;

/**
 * Provides some extra mathematical functions not present in the standard Java API.
 *
 * @author Mathijs den Burger
 */

public class ExtraMath {

	/**
	 * Checks if the given integer is odd.
	 * @return true if the given integer is odd, false otherwise.
	 */
	public static boolean odd(int i) {
		if ((Math.ceil(i / 2) * 2) == i) return false;
		else return true;
	}

	/**
	 * Checks if the given integer is even.
	 * @return true if the given integer is even, false otherwise.
	 */
	public static boolean even(int i) {
		return !odd(i);
	}

	/**
	 * Clamps the given value, that is: force it into a given range.
	 * This means:<BR>
	 * <UL>
	 * <LI>if value < min then result = min
	 * <LI>if value > max then result = max
	 * <LI>else result = value
	 * </UL>
	 * @return The clamped value
	 */
	public static int clamp(int value, int min, int max) {
		return Math.min(Math.max(value, min), max);
	}
	
	/**
	 * Checks if the given value is dividable by the given divisor.
	 * (e.g. 4 and 6 are dividable by 2, but 5 is not).
	 * @param value The value to divide
	 * @param divisor The number to divide by
	 * @return true if the value is dividable by the given divisor, false otherwise
	 */
	public static boolean dividableBy(int value, int divisor) {
		float valueFloat = (float)value, divisorFloat = (float)divisor;
		return ((Math.floor(valueFloat / divisorFloat) * divisorFloat) == valueFloat);
	}
	
}