package quest.global.util;

import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.*;

/**
 * This class extends {@link java.util.Observable}
 * by making it serializable. No other functionality is added.
 *
 * @see java.util.Observable
 * @see	java.io.Serializable
 * @author Mathijs den Burger
 */
public class SerializableObservable extends Observable implements Serializable {

    private Vector observers;

    public SerializableObservable() {
	super();
	observers = new Vector();
    }
	
    public synchronized void addObserver(Observer o) {
	if (!observers.contains(o)) {
	    super.addObserver(o);
	    observers.addElement(o);
	}
    }

    public synchronized void deleteObserver(Observer o) {
        super.deleteObserver(o);
        observers.removeElement(o);
    }
    
    public synchronized void deleteObservers() {
	super.deleteObservers();
	observers.removeAllElements();
    }
    
    private void writeObject(ObjectOutputStream out) throws IOException {
	out.defaultWriteObject();
    }

    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
	in.defaultReadObject();
	for (int i = 0; i < observers.size(); i++) {
	    super.addObserver((Observer)(observers.elementAt(i)));
	}
    }	
 
}
