package quest.server;

import quest.server.chat.*;
import quest.server.net.*;
import quest.server.login.*;
import quest.server.log.*;
import quest.server.util.MYSQLCommunicator;
import quest.server.util.QuestDatabase;
import quest.server.game.*;

/**
 * Runs the server. This starts all servers, like the login, chat and game server.
 * It initializes the communication with the database and creates the logging 
 * mechanism for the server status and error messages.
 *
 * @author    Merijn Evertse
 */
public class Server {

  private LogManager log;

	/**
	 * Creates the server.
	 */
	Server() {
	    log = LogManager.getInstance();
	    log.writeEntry("Server: Starting up");
		
		// start login controller and database communications
		new MYSQLCommunicator("localhost", "groep98", "groep98", "m1m2s3");
		new QuestDatabase();
		new LoginController();

		// start server daemon for incoming connections
	    ServerDaemon.getInstance();

		
		// initiate servers
		new GameServer(); // chat needs game
		new ChatServer();

		log.writeEntry("Server: Up and running now, have fun");
	}

  /**
   * Starts the server application.
   *
   * @param    args    The arguments for the server.
   *                   NOTE: at the moment there are no arguments possible.
   */
	public static void main(String args[]) {
		new Server();
	}
}
