package quest.server.chat;

import java.util.*;

/**
 * Represents a user of the chat system
 *
 * @author    Merijn Evertse
 */
class ChatUser {

    /**
     * User is in a channel where no game can be started.
     */
    public static final int USERMODE_UNDEFINED = 0;
    
    /**
     * User is a player.
     */
    public static final int USERMODE_PLAYER = 1;
    
    /**
     * User is a player and is the administrator of this channel.
     */
    public static final int USERMODE_PLAYERADMIN = 2;
    
    /**
     * User is currently a spectator but wants to be a player.
     */
    public static final int USERMODE_SPEC_PLAYER = 3;
    
    /**
     * User is a spectator.
     */
    public static final int USERMODE_SPEC_SPEC = 4;
    
    /**
     * The current modes of the user for each channel the user is in.
     */
    private Vector channelUsermodes;
    
    /**
     * The name of the user.
     */
    public String username;

    /**
     * Creates a new user without values.
     */
    ChatUser() {
	System.out.println("User gemaakt zonder waarden");
	username = "";
	channelUsermodes = new Vector();
    }
    
    /**
     * Creates a new user with the given username and the given
     * usermode.
     *
     * @param    username    The name of the new user
     * @param    usermode    The mode of the new user
     */
    public ChatUser(String username) {
	System.out.println("User gemaakt met waarden " + username);
	this.username = username;
	channelUsermodes = new Vector();
    }
    
    /**
     * Adds the user to the given channel as a client with the given usermode.
     *
     * @param    channelname    The name of the channel
     * @param    usermode       The mode of the user in the channel
     */
    public void subscribe(String channelname, int usermode) {
	System.out.println("Subscribe " + username + " to " + channelname + " with mode " + usermode); 
	channelUsermodes.addElement(
		new ChannelUsermode(channelname, usermode));
    }
    
    /**
     * Removes a user from the given channel.
     *
     * @param    channelname    The name of the channel
     */
    public void unsubscribe(String channelname) {
	System.out.println("Unsubscribe " + username + " to " + channelname); 
	channelUsermodes.removeElement(
		new ChannelUsermode(channelname, USERMODE_UNDEFINED));
    }
    
    /**
     * Returns the usermode of the client in the given channel.
     *
     * @param     channelname    The name of the channel
     * @return    The usermode in the channel, or USERMODE_UNDEFINED if the
     *	 	      user doesn't have a subscribtion to the channel.
     */
    public int getUsermode(String channelname) {
	  if (channelname != null && !(channelname.equals(Channel.NOCHANNEL))) { 
		System.out.println("Give me the usermode for " + username + " in channel " + channelname);
		return find(channelname).usermode;
	  }
	  return USERMODE_UNDEFINED;
	}

    /**
     * Sets the usermode for the given channel to the given mode.
     *
     * @param    channelname    The name of the channel
     * @param    usermode	The mode to set the usermode to
     * @return   TRUE:    if changing usermode was allowed
     *		 FALSE:   if not
     */
    public boolean setUsermode(String channelname, int usermode) {
	int currentMode = find(channelname).usermode;
	System.out.println("Changing " + username + " from " + currentMode + " to " + usermode);
	// FIXME: ff checken of het wel mag
	find(channelname).usermode = usermode;

	return true;
    }

  /**
   * Returns a list of all the channels the user is in.
   *
   * @return    The list of channels
   */
  public Vector getChannelnames() {
	Vector channelnames = new Vector();
	for (int i = 0; i < channelUsermodes.size(); i++) {
	  channelnames.addElement(((ChannelUsermode) channelUsermodes.elementAt(i)).channelname);
	}
	return channelnames;
  }

  /**
   * Returns the first channel, after the main chat channel, the user is in.
   *
   * @return    The channel the user is in
   */
  public String getChannelname() {
	Vector cn = getChannelnames();
	if (cn.size() >= 1) {
	  return (String) cn.elementAt(cn.size() - 1);
	} else {
	  return Channel.NOCHANNEL;
	}
  }

  private ChannelUsermode find(String channelname) {
	for (int i = 0; i < channelUsermodes.size(); i++) {
	  if (((ChannelUsermode) channelUsermodes.elementAt(i)).channelname.equals(channelname)) {
		return (ChannelUsermode) channelUsermodes.elementAt(i);
	  }
	}
	return new ChannelUsermode(null, USERMODE_UNDEFINED);
  }

  /**
   * Removes a channel from the list of channels the user is in.
   *
   * @param    channelname    The name of the channel
   */
  public void removeChannel(String channelname) {
	for (int i = 0; i < channelUsermodes.size(); i++) {
	  if (((ChannelUsermode) channelUsermodes.elementAt(i)).channelname.equals(channelname)) {
		channelUsermodes.removeElementAt(i);
	  }
	}
  }
}
