package quest.server.chat;

import java.util.*;

/**
 * This is the database of all currently online users of the chatsystem.
 * It also provides all methodes for working with these users
 *
 * @author    Merijn Evertse
 */
public class ChatUsers {

  private static Hashtable users;

  /**
   * Initializes the database of users.
   */
  public ChatUsers() {
    users = new Hashtable();
  }

  /**
   * Adds a user to the user database, because the user is now online.
   *
   * @param    user        The user to add
   */
  public void addUser(ChatUser user) {
	removeUser(user);
    users.put(user.username, user);
  }

  /**
   * Removes the user from the user database, because the user went offline.
   *
   * @param    user        The user to remove
   */
  public void removeUser(ChatUser user) {
    users.remove(user.username);
  }

  /**
   * Returns the user with the given username.
   *
   * @param    username    The name of the user
   * @return   The user
   */
  public ChatUser getUser(String username) {
      if (username != null) 
	  return (ChatUser) users.get(username);
      return null;
  }

  /**
   * Returns whether the user is online or not.
   *
   * @param    username    The name of the user
   * @return   TRUE:  if the user is online
   *           FALSE: if not
   */
  public boolean userExists(String username) {
	if (username != null)
	  if (((ChatUser) users.get(username)) != null)
		return true;
	return false;
  }

  /**
   * Removes a channel from the list of channels the user is in.
   *
   * @param    username    The name of the user
   * @param    channelname The name of the channel to remove
   */
  public void removeChannel(String username, String channelname) {
	if (username != null) {
	  ChatUser user = (ChatUser) users.get(username);
	  if (user != null)
		user.removeChannel(channelname);
	}
  }
}
