package quest.server.log;

import java.io.*;
import java.util.*;
import java.text.*;
import quest.server.net.*;
import quest.global.file.*;

/**
 * Handles writing to the server logfile. This way it is simple to ensure that
 * all log entries are in a normalised form and access to the file is restricted to
 * one access point.
 *
 * @author    Merijn Evertse
 */
public class LogManager {

  private static LogManager uniqueInstance = null;
  private static GregorianCalendar calendar;
  private static SimpleDateFormat formatter;
  private static FileStream stream;
  private static File file;
  
  protected LogManager() {
	calendar = new GregorianCalendar();
	formatter = new SimpleDateFormat("MMM d z yyyy, hh:mm:ss");
	file = new File(ServerConfiguration.getInstance().getLogFilename());
	stream = new FileStream(file);
  }
  
  /**
   * Write the given entry to the log.
   *
   * @param    entry    The new log entry
   */
  public void writeEntry(String entry) {
	Date date = calendar.getTime();	
	
	stream.write(formatter.format(calendar.getTime()) +
				 " : " +
				 entry);
	System.out.println("LOG: " + entry);
  }
  
  /**
   * Returns an instance of this class, making sure only one object of it exists.
   *
   * @return    A reference to the instance of this class
   */
  public static LogManager getInstance() {
  	if (uniqueInstance == null) {
	  uniqueInstance = new LogManager();
  	}
	return uniqueInstance;
  }
}
