package quest.server.net;

import java.io.*;
import java.net.*;
import quest.global.net.*;

/**
 * The daemon which listens to the server port for incoming connections of clients who want to
 * login to the system.
 * Starts a registration procedure for authenticating the client and putting the client online.
 *
 * @author    Merijn Evertse
 */
public class ListenDaemon extends Thread {

	private ServerDaemon server;
	
	/**
	 * Initializes the listen daemon to listen on the given port and use the given server as
	 * database for the user.
	 */
	public ListenDaemon() {
		server = ServerDaemon.getInstance();
	}
	
	/**
	 * Runs this thread.
	 */
	public void run() {
		try {
			ServerSocket serverSocket = new ServerSocket(ServerConfiguration.getInstance().getServerPort());

			while(true) {
				final Socket connection = serverSocket.accept();
				
				Runnable ra = new Runnable() {
					public void run() {
					MessageStream messageStream = new MessageStream(connection, MessageStream.SERVER_SIDE);
					// use messageStream.toString as temporary unique identifier for user, until authenticated
					server.registerUser(messageStream.toString(), messageStream);
					}
					};
				(new Thread(ra)).start();
			}
		} catch (IOException e) {
			System.out.println(e.getMessage());
		}
	}
}	


