package quest.server.net;

import java.net.*;
import quest.global.net.*;

import java.util.Vector;

/**
 * Represents a user of the communication system.
 *
 * @author    Merijn Evertse
 */
public class NetUser {

  /**
   * The username of this user.
   */
  public String username;
  
  /**
   * The input and output message stream between the user and the
   * server.
   */
  public MessageStream messageStream;
  
  private Vector enabledProtocols;
  
  /**
   * Creates a new user without values.
   */
  NetUser() {
    username   = "";
    messageStream = null;
	enabledProtocols = new Vector();
  }
  
  /**
   * Creates a new user with the given username and the given
   * message stream.
   *
   * @param    username         The name of the new user
   * @param    messageStream    The message stream with the server
   */
  public NetUser(String username, MessageStream messageStream) {
    this.username      = username;
    this.messageStream = messageStream;
	enabledProtocols = new Vector();
  }

  /**
   * Subscribes a user to a message protocol.
   *
   * @param    messageType    The message protocol to subscribe to
   */
  public void subscribe(Class messageType) {
	enabledProtocols.addElement(messageType);
  }

  /**
   * Returns whether a user is subscribed to the given message protocol.
   *
   * @param    messageType    The message protocol to check for
   */
  public boolean subscribedTo(Class messageType) {
	for (int i = 0; i < enabledProtocols.size(); i++) {
	  if (((Class) enabledProtocols.elementAt(i)).isAssignableFrom(messageType)) {
		return true;
	  }
	}
	return false;
  }

  /**
   * Unsubscribes the user from the message protocol.
   *
   * @param    messageType    The message protocol to unsuubscribe from
   */
  public void unsubscribe(Class messageType) {
	enabledProtocols.removeElement(messageType);
  }
}
