package quest.server.net;

import java.util.*;
import quest.global.net.*;

/**
 * Contains the user database for the communication system.
 *
 * @author    Merijn Evertse
 */
public class NetUsers {

	protected static NetUsers uniqueInstance;
	
	private Hashtable users;

	/**
	 * Creates an empty user database.
	 */
	protected NetUsers() {
		users = new Hashtable();
	}
	
	/**
	 * Adds a user to the database.
	 *
	 * @param    user    The user to add
	 */
	public void addUser(NetUser user) {
		users.put(user.username, user);
	}

	/**
	 * Removes a user from the database.
	 *
	 * @param    user    The user to remove
	 */
	public void removeUser(NetUser user) {
		users.remove(user.username);
	}

	/**
	 * Returns the user with the given username.
	 *
	 * @param    username    The name of the user
	 * @return   The user
	 */
	public NetUser getUser(String username) {
		NetUser user = (NetUser) users.get(username);
		if (user != null) System.out.println(user.username + "user gevonden");
		return user;
}
	
	/**
	 * Returns the message streams associated with the given usernames.
	 *
	 * @param    usernames   The usernames to get the message streams for
	 * @return   The message streams associated with the usernames
	 */
	public Vector getStreams(Vector usernames) {
		Vector messageStreams = new Vector();
		for(int i = 0; i < usernames.size(); i++) {
			String username = (String) usernames.elementAt(i);
			System.out.println("get stream from user: " + username);
			if (username != null) {
			  NetUser user = getUser(username);
			  if (user != null) messageStreams.addElement(user.messageStream);
			}
		}
		return messageStreams;
	}
	
	/**
	 * Returns an enumeration of all users.
	 *
	 * @return    All users
	 */
	public Enumeration getUsers() {
		return users.elements();
	}
	
	/**
	 * Returns an enumeration of all users subscribed to the given message type.
	 *
	 * @param    messageType    The type of the message to check for
	 * @return   All usernames of the users subscribed to the message type
	 */
	public Vector getSubscribedUsers(Class messageType) {
		Enumeration allusers = users.elements();
		Vector subscribedusers = new Vector();
		
		for (; allusers.hasMoreElements();) {
			NetUser user = (NetUser) allusers.nextElement();
			if (user.subscribedTo(messageType)) subscribedusers.addElement(user.username);
		}
		return subscribedusers;
	}
	
	/**
	 * Returns an instance of this class, making sure only one object of it exists.
	 *
	 * @return    A reference to the instance of this class
	 */
	public static NetUsers getInstance() {
		if (uniqueInstance == null) {
			uniqueInstance = new NetUsers();
		}
		return uniqueInstance;
	}
}
