package quest.server.net;

import java.net.*;

/**
 * Stores the configuration of the server.
 * Used as a central point for all configuration settings.
 *
 * @author    Merijn Evertse
 */
public class ServerConfiguration {

	private InetAddress serverAddress;
	private int serverPort;
	private int sleeptime;  // in milliseconds
	private static ServerConfiguration uniqueInstance = null;

	/**
	 * Initializes some variables so they can be uses troughout the system.
	 */
	protected ServerConfiguration() {
		try {
			serverAddress = InetAddress.getByName("localhost");
		} catch (UnknownHostException e) {
			System.out.println(e.getMessage());
		}
		serverPort  = 5000;
		sleeptime = 5;
	}
	
	/**
	 * Returns the internet address of the server.
	 *
	 * @return    The InetAddress of the server
	 */
	public InetAddress getServerAddress() {
		return serverAddress;
	}
	
	/**
	 * Returns the port number the listening daemon should run on.
	 *
	 * @return    The listening server port
	 */
	public int getServerPort() {
		return serverPort;
	}
	
	/**
	 * Returns the amount of time (in milliseconds) the server should wait between requests
	 * and actions. This is to prevent some server actions from using too much cputime.
	 *
	 * @return    The sleeptime in milliseconds
	 */
	public int getSleeptime() {
		return sleeptime;
	}

	/**
	 *  Returns the name of the file the log should be written in.
	 *
	 * @return    The log filename
	 */
	public String getLogFilename() {
		return "logfile";
	}

	/**
	 * Returns an instance of this class, making sure only one object of it exists.
	 *
	 * @return    A reference to the instance of this class
	 */
	public static ServerConfiguration getInstance() {
		if (uniqueInstance == null) {
			uniqueInstance = new ServerConfiguration();
		}
		return uniqueInstance;
	}
}
